package gs.hex.v0

import java.security.SecureRandom

class HexTests extends munit.FunSuite:

  val Rng       = new SecureRandom()
  val MinLength = 1
  val MaxLength = 100

  test("should convert an empty array to the empty string") {
    assert(Hex.toHexString(Array.empty) == "")
  }

  test("should convert an empty string to an empty array") {
    assert(Hex.fromHexString("").map(_.toList) == Some(List.empty[Byte]))
  }

  test(
    "(failure case) should convert a string with an uneven length to nothing"
  ) {
    assert(Hex.fromHexString("0") == None)
    assert(Hex.fromHexString("000") == None)
    assert(Hex.fromHexString("00000") == None)
    assert(Hex.fromHexString("0000000") == None)
    assert(Hex.fromHexString("000000000") == None)
  }

  test(
    "(failure case) should produce nothing if invalid characters are present"
  ) {
    assert(Hex.fromHexString("0!") == None)
    (0 to 255).foreach { ch =>
      if !isValidHexChar(ch) then
        assert(Hex.fromHexString("0" + ch.toChar.toString()) == None)
      else ()
    }
  }

  test("should round trip arrays of bytes") {
    (0 to 10).foreach { _ =>
      val originalData = randomByteArray()
      val encoded      = Hex.toHexString(originalData)
      val decoded      = Hex.fromHexString(encoded)
      assert(decoded.isDefined)
      assert(decoded.map(_.sameElements(originalData)).getOrElse(false))
    }
  }

  private def isValidHexChar(numeral: Int): Boolean =
    (numeral >= 48 && numeral <= 57)
      || (numeral >= 65 && numeral <= 90)
      || (numeral >= 97 && numeral <= 122)

  private def randomByteArray(): Array[Byte] =
    val length = Rng.nextInt(MaxLength) + MinLength
    val bytes  = new Array[Byte](length)
    val _      = Rng.nextBytes(bytes)
    bytes
